function [cwardKrate,cwardSrate,cwardJrate] = cwardInferenceFast(kappa,trueS,sampleSize,simSize,alpha)
    %------------------------------------------------------------------------------------------
    %   Description:
    %   	An optimized version of cwardInference.
    %   Synopsis:
    %       [cwardKrate,cwardSrate,cwardJrate] = cwardInferenceFast(kappa,trueS,sampleSize,simSize,alpha)
    %   Output:
    %       cwardKrate = coverate rate for kappa based on LRT
    %       cwardSrate = coverage rate for s based on LRT
    %       cwardJrate = joint coverage rate for k and s based on LRT
    %------------------------------------------------------------------------------------------
    %check arguments
    if nargin<4
        simSize = 1000;
        alpha = 0.05;
    elseif nargin<5
        alpha = 0.05;
    end
    cwardKrate = zeros(simSize,1);
    cwardSrate = zeros(simSize,1);
    cwardJrate = zeros(simSize,1);
    kappaString = num2str(kappa);
    sizeString = num2str(sampleSize);
    suffix = strcat('_', kappaString, '_', sizeString);
    folder = strcat('mcmc', suffix);
    %read in mle data
    mleZXZFileName = strcat(folder, '/mleZXZ', suffix, '.txt');
    mleZXZ = dlmread(mleZXZFileName);
    %read in hessian matrices
    hessianZXZFileName = strcat(folder, '/hessiansZXZ', suffix, '.bin');
    hessianZXZ = readHessian(hessianZXZFileName);
    parfor i = 1:simSize
        %inference for kappa
        mle = mleZXZ(i,:);
        kMLE = mle(2);
        hessian = hessianZXZ(:,:,i);
        invCovMatrix = -hessian;
        covMatrix = inv(invCovMatrix);
        dk = kMLE - kappa;
        significant = chisqTest(dk, covMatrix(1,1), false, alpha);
        if ~significant
            cwardKrate(i) = 1;
        end
        %inference for s and k-s together
        eulerMLE = mle(3:5);
        trueSClass = equiClass(trueS);
        dEulerClass = zeros(24, 3);
        for count = 1:24
            [tr1, tr2, tr3] = dcm2angle(trueSClass(:,:,count), 'zxz', 'ZeroR3');
			dEulerClass(count, :) = eulerMLE - [tr1, tr2, tr3];
        end
        for count = 1:24
			significant = chisqTest(dEulerClass(count, :), covMatrix(2:4, 2:4), false, alpha);
			if ~significant 
				cwardSrate(i) = 1;
				break;
			end
        end
        for count = 1:24
			significant = chisqTest([dk,dEulerClass(count, :)], invCovMatrix, true, alpha);
			if ~significant
				cwardJrate(i) = 1;
				break;
			end
        end
    end
    % coverage rate 
    cwardKrate = mean(cwardKrate);
    cwardSrate = mean(cwardSrate);
    cwardJrate = mean(cwardJrate);
    % write results into a file 
    cwardRateFileName = strcat(folder,'/correctedWardRates_',kappaString,'_',sizeString,'.txt');
    cwardRateFileId = fopen(cwardRateFileName,'w');
    fprintf(cwardRateFileId,'cwardKrate: %f, cwardSrate: %f, cwardJrate: %f',cwardKrate,cwardSrate,cwardJrate);
    fclose(cwardRateFileId);
