function [kcr, scr, kCenter, sCenter, jcr] = ksciPar(sTruth, kappa, sampleSize, dropSize, simSize, alpha)
    %   Calculate confidence interval coveragte rate for kappa and s.
    %
    %   Synopsis:
    %       [kcr, scr, kCenter, sCenter, jcr] = ksciPar2(sTruth, kappa, sampleSize, dropSize, simSize, alpha)
    %   Input:
    %       sTruth = the true value of s. 
    %       kappa = value of kappa used in the simulation.
    %       sampleSize = sample size used in the simulation.
    %       dropSize = the burn-in size.
    %       simSize = the number of mcmc samples.
    %       alpha = significant level.
    %   Output:
    %       kcr = coverage rate of the center, upper limit and lower limit confidence intervals for kappa.
    %       scr = coverage rate for s using kappa from data. 
    %       kCenter = center kappa vector from data.
    %       sCenter = center s vector using kappa from data.
    %check passing of parameter
    if nargin < 6
       alpha = 0.05; 
    end
    %--------------- initialize variables ------------------
    %coverage
    kcrCenter = zeros(1, simSize);
    kcrUpper = zeros(1, simSize);
    kcrLower = zeros(1, simSize);
    scr = zeros(1, simSize);
    jcr = zeros(1, simSize);
    %center parameters used in the inferences
    kCenter = zeros(1, simSize);
    sCenter = zeros(3, 3, simSize);
    %error subject
    kappaString = num2str(kappa);
    sizeString = num2str(sampleSize);
    suffix = strcat('_', kappaString, '_', sizeString);
    folder = strcat('mcmc', suffix);
    errSubject = strcat('Exception Happens in Inference for k=', kappaString, ', n=', sizeString);
    %do inference
    parfor i = 1:simSize
       iString = num2str(i);
       %read in a data set of k
       kFileName = strcat(folder, '/kpost/kpost', suffix, '_', iString, '.bin');
       [kpost, kJumpRate] = readPostK(kFileName, dropSize);
       %do inference for kappa based on information of kappa only
       kQuan = quantile(kpost, [alpha/2, 1-alpha/2, alpha, 1-alpha]);
       if kappa > kQuan(1) && kappa < kQuan(2)
           kcrCenter(i) = 1;
       end
       if kappa > kQuan(3)
           kcrLower(i) = 1;
       end
       if kappa < kQuan(4)
           kcrUpper(i) = 1;
       end
       %read in data set of s
       sFileName = strcat(folder, '/spost/spost', suffix, '_', iString, '.bin');
       [spost, sJumpRate] = readPostS(sFileName, dropSize);
       %read in rotation matrix data
       rmFileName = strcat(folder, '/rms/rms', suffix, '_', iString, '.bin');
       rmSam = readRMS(rmFileName);
       %------------------- do inference for s and kappa ------------------
       %read in log posteriors 
       lpsFileName = strcat(folder, '/lps/lps', suffix, '_', iString, '.bin');
       lps = readLPS(lpsFileName, dropSize);
       %---------- highest posterior inference ----------------
       %5% quantile of log posteriors
       lpsLowerLimit = quantile(lps, alpha);
       if logpost(sTruth, kappa, rmSam) > lpsLowerLimit
          jcr(i) = 1; 
       end
       %find max log posterious
       [lpsMax, lpsIndex] = max(lps);
       sCenter(:, :, i) = spost(:, :, lpsIndex);
       kCenter(i) = kpost(lpsIndex);
       %--- distance between draws of s and the center of s ---
       %using posterior mode for s where truth for kappa is used
       %the real posterior mode for s, i.e. kappa is from posterior draws
       lenUsed = numel(kpost);
       sDistance = zeros(1, lenUsed);
       pmodeClass = equiClass(spost(:, :, lpsIndex));
       for j = 1:lenUsed
          sDistance(j) = minmaxAngle(spost(:,:,j), pmodeClass);
       end
       %------ write the distances into files ----------------
       %distances using kappa from posterior draws
       sDistFileName = strcat(folder, '/sDist', suffix, '_', iString, '.bin');
       safeWriteBinary(sDistFileName, 'double', errSubject, sDistance);
       %calculate quntile of the distances
       sDistUpperLimit = quantile(sDistance, 1-alpha);
       %calculate the distance between the truth of s and the posterior
       %modes
       if minmaxAngle(sTruth, pmodeClass) < sDistUpperLimit
          scr(i) = 1; 
       end
    end
    %calculate coverage rate
    kcr = [mean(kcrCenter), mean(kcrUpper), mean(kcrLower)];
    scr = mean(scr);
    jcr = mean(jcr);
    %--------------------------- write result into files -----------------
    %write kcRate and sfcRate into a text file
    covRateFileName = strcat(folder, '/covRate', suffix, '.txt');
    covRateFileId = fopen(covRateFileName, 'w');
    fprintf(covRateFileId, 'kcrCenter: %f, kcrUpper: %f, kcrLower: %f, scr: %f, jcr: %f', kcr, scr, jcr);
    fclose(covRateFileId);
    %write sCenter into a file
    sCenterFileName = strcat(folder, '/sCenter', suffix, '.bin');
    safeWriteBinary(sCenterFileName, 'double', errSubject, sCenter);
    %write kCenter into a file
    kCenterFileName = strcat(folder, '/kCenter', suffix, '.bin');
    safeWriteBinary(kCenterFileName, 'double', errSubject, kCenter);
    %report progress via email
    emailSubject = strcat('100% of inference for k=', kappaString, ', n=', sizeString, ' is done!');
    emailBody = strcat('Inference results: kcr=', num2str(kcr), ', scr=', num2str(scr), ', jcr=', num2str(jcr), ', jcr=', num2str(jcr));
    disp(emailSubject);
    disp(emailBody);
    emailMe(emailSubject, emailBody);
