function [kpost,spost,krate,srate,tuningK,tuningS] = mcmcSample(n,sInit,kInit,dataOm,tuningSK,adjust)
%%
%   Make posterior draws for parameters in the Matrix Fisher distribution.
%
% Synopsis:
%   [kpost,spost,krate,srate,tuningK,tuningS] = mcmcSample(n,sInit,kInit,dataOm,tuningSK,adjust)
% 
%   Input:
%       n = the number of draws to make.
%       sInit = the initial value for s.
%       kInit = the initial value for k.
%       dataOm = the rotation matrix data.
%       tuningSK = a 1*2 matrix which contains the tuning parameter for S and K.
%       adjust = a 1*10 vector of the form [minin,iter,sjpLeast,sjpMost,sAcceptChange,sRejectChange,kjpLeast,kjpMost,kAcceptChange,kRejectChange] 
%       to control the automation of tuning
%       parameters. It means that we use at least nLeast steps and at most
%       nMost steps to adjust the tuning parameters. And the final choice
%       of tuning parameters should make the jumping rate of s falls into
%       interval (sjpLeast, sjpMost), and make the jumping rate of k falls into
%       interval (kjpLeast, kjpMost). If the jumping rates never falls into the
%       two intervals at the same time between the nLeast and nMost steps, the
%       program will pause and ask user intervention; otherwise, the
%       program will report the tuning parameters and jumping rate and number of steps used, then
%       continue to make n draws. If adjust is not specified, then the
%       tuning parameters will not be adjusted. 
%
%   Output:
%       kpost = posterior draws for kappa, which is a vector.
%       spost = posterior draws for s, which is 3 dimensional array.
%       krate = the jumping rate for kappa, which can also be abtained from
%       the posterior draws for kappa.
%       srate = the jumpint rate for s, which can also be abtained from the
%       posterior draws for s.
% 
    
        sCurrent=sInit;
        kCurrent=kInit;
        tuningS = tuningSK(1);
        tuningK = tuningSK(2);
    if nargin<6 %no tuning going on
        %extract initial values and tuning parameters for s and k
        krate=0;
        srate=0;
        kpost=zeros(1,n);
        spost=zeros(3,3,n);
        for i = 1:1:n
            %generate a new s
            sStar = rmfSymOne(sCurrent,tuningS);
            r1=ratioAcceptS(sStar,sCurrent,kCurrent,dataOm);
            %check whether should jump
            if r1>=1
                spost(:,:,i)=sStar;
                sCurrent=sStar;
                srate=srate+1;
            elseif binornd(1,r1)==1
                spost(:,:,i)=sStar;
                sCurrent=sStar;
                srate=srate+1;
            else
                spost(:,:,i)=sCurrent;
            end
            %generate a new k
            kStar=lognrnd(log(kCurrent),tuningK);
            r2=ratioAcceptK(sCurrent,kStar,kCurrent,dataOm);
            if r2>=1
                kpost(i)=kStar;
                kCurrent=kStar;
                krate=krate+1;
            elseif binornd(1,r2)==1
                kpost(i)=kStar;
                kCurrent=kStar;
                krate=krate+1;
            else
                kpost(i)=kCurrent;
            end
        end
        %%
        %calculate the jumping rate
        srate=srate/n;
        krate=krate/n;
        return;
    end
    minin = adjust(1);
    iter = adjust(2);
    tuningChangeRatio = adjust(3);
    sjpLeast = adjust(4);
    sjpMost = adjust(5);
    kjpLeast = adjust(6);
    kjpMost = adjust(7);
    %automate tuning parameters
    kpost=zeros(1,minin);
    spost=zeros(3,3,minin);
    srate = (sjpLeast+sjpMost)/2;
    krate = (kjpLeast+kjpMost)/2;
    for i = 1:1:iter
        %adjust tuning parameters and initial values
        if krate<kjpLeast 
            tuningK = tuningK/tuningChangeRatio;
        end
        if krate>kjpMost
           tuningK = tuningK*tuningChangeRatio; 
        end
        if srate<sjpLeast
           tuningS = tuningS*tuningChangeRatio;
        end
        if srate>sjpMost
           tuningS = tuningS/tuningChangeRatio; 
        end
        srate = 0;
        krate = 0;
        for j = 1:1:minin
        %generate a new s
            sStar = rmfSymOne(sCurrent,tuningS);
            r1=ratioAcceptS(sStar,sCurrent,kCurrent,dataOm);
            %check whether should jump
            if r1>=1
                spost(:,:,j) = sStar;
                sCurrent = sStar;
                srate = srate + 1;
            elseif binornd(1,r1)==1
                spost(:,:,j) = sStar;
                sCurrent=sStar;
                srate=srate+1;
            else
                spost(:,:,j) = sCurrent;
            end
            %generate a new k
            kStar=lognrnd(log(kCurrent),tuningK);
            r2=ratioAcceptK(sCurrent,kStar,kCurrent,dataOm);
            if r2>=1
                kpost(j) = kStar;
                kCurrent=kStar;
                krate=krate+1;
            elseif binornd(1,r2)==1
                kpost(j) = kStar;
                kCurrent=kStar;
                krate=krate+1;
            else
                kpost(j) = kCurrent;
            end
        end
        krate = krate/minin;
        srate = srate/minin;
        if krate>=kjpLeast && krate<=kjpMost && srate>=sjpLeast && srate<=sjpMost
            %report current tuning parameters
            disp('Tuning Succeds!');
            disp(strcat('Mini-Steps: ',num2str(minin),', Iterations: ', num2str(i)));
            disp(strcat('TuningS: ', num2str(tuningS),', sjRate: ',num2str(srate),', TuningK: ',num2str(tuningK),', kjRate: ',num2str(krate)));
%             disp('Initial value for S:');
%             disp(sInit);
%             disp('Initial value for Kappa:');
%             disp(kInit);
            [kpost,spost,krate,srate,tuningK,tuningS] = mcmcSample(n,sInit,kInit,dataOm,[tuningS,tuningK]);
            return;
        end 
        %print current information out
         disp('Tuninging ...');
         disp(strcat('Mini-Steps: ',num2str(minin),', Iterations: ', num2str(i)));
         disp(strcat('TuningS: ', num2str(tuningS),', sjRate: ',num2str(srate),', TuningK: ',num2str(tuningK),', kjRate: ',num2str(krate)));
    end
    %no good tuning parameter is found, ask for user intervention
    disp('Tuning Fails!');
    disp(strcat('Mini-Steps: ',num2str(minin),', Iterations: ', num2str(iter)));
    disp(strcat('TuningS: ',num2str(tuningS),', sjRate: ',num2str(srate),', TuningK: ',num2str(tuningK),', kjRate: ',num2str(krate)));
    disp('Please make a choice:');
    disp('R/r: Re-tuning.');
    disp('I/i: Ignore failure and run MCMC');
    disp('Q/q: Quit.');
    userIntent = input('','s');
    userIntent = lower(userIntent);
    if userIntent == 'r'
        [kpost,spost,krate,srate,tuningK,tuningS] = mcmcSample(n,sInit,kInit,dataOm,[tuningS,tuningK],adjust);
        return;
    elseif userIntent == 'i'
        [kpost,spost,krate,srate,tuningK,tuningS] = mcmcSample(n,sInit,kInit,dataOm,[tuningS,tuningK]);
        return;
    end
    
