
function [kpost,spost,krate,srate] = mcmcSample_or(n,sInit,kInit,dataOm,tuningS,tuningK)
%%
% Synopsis:
%   [kpost, spost, krate, srate] = mcmcSample_or(n,sInit,kInit,dataOm,tuningS,tuningK)
% 
%   Input:
%       n = the number of draws to make.
%       sInit = the initial value for s.
%       kInit = the initial value for k.
%       dataOm = the rotation matrix data.
%      
%
%   Output:
%       kpost = posterior draws for kappa, which is a vector.
%       spost = posterior draws for s, which is 3 dimensional array.
%       krate = the jumping rate for kappa, which can also be abtained from
%       the posterior draws for kappa.
%       srate = the jumpint rate for s, which can also be abtained from the
%       posterior draws for s.
    %%
    %initialize the output
    kpost=zeros(1,n);
    spost=zeros(3,3,n);
    krate=0;
    srate=0;
    %%
    %extract initial values for s and k
    sCurrent=sInit;
    kCurrent=kInit;
    %%
    %do mcmc simulation accroding to the the number of tuning parameters
        %%
        %--- mcmc simulation using fixed tuning parameters ------
        for i = 1:n
            %generate a new s
            sStar = rmfSymOne(sCurrent,tuningS);
            r1=ratioAcceptS_or(sStar,sCurrent,kCurrent,dataOm);
            %check whether should jump
            if r1>=1
                spost(:,:,i)=sStar;
                sCurrent=sStar;
                srate=srate+1;
            elseif binornd(1,r1)==1
                spost(:,:,i)=sStar;
                sCurrent=sStar;
                srate=srate+1;
            else
                spost(:,:,i)=sCurrent;
            end
            %generate a new k
            kStar=lognrnd(log(kCurrent),tuningK);
            r2=ratioAcceptK_or(sCurrent,kStar,kCurrent,dataOm);
            if r2>=1
                kpost(i)=kStar;
                kCurrent=kStar;
                krate=krate+1;
            elseif binornd(1,r2)==1
                kpost(i)=kStar;
                kCurrent=kStar;
                krate=krate+1;
            else
                kpost(i)=kCurrent;
            end
        end
        %%
        %calculate the jumping rate
        srate=srate/n;
        krate=krate/n;

function y = ratioAcceptK_or(s, kStar, k, dataOm)
    %%
    %   Synopsis:
    %       y = ratioAcceptK_dis(s, kStar, k, dataOm)
    %   Input:
    %       s = the current s which is the center rotation matrix.
    %       kStar = a proposal for k.
    %       k = the current value for kappa which is the parameter for the
    %       random angle distribution.
    %       dataOm = the rotation matrix data.
    %   Output:
    %       y = the ratio for accepting the kStar which is a proposal for
    %       k.
    %%
    %calculate intermediate results
    %scaled bessel function for 2*kStar
    i02kStarExp=besseli(0,2*kStar,1);
    i12kStarExp=besseli(1,2*kStar,1);
    %scaled bessel function for 2*k
    i02kExp=besseli(0,2*k,1);
    i12kExp=besseli(1,2*k,1);
    %%
    %calculate the first part of log ratioAcceptK_dis
    %get the dim of dataOm
    n=size(dataOm,3);
    if(k>150)
        temp = 3*k-log(4*k)-0.5*log(4*pi*k);
    else
        temp = i02kExp - i12kExp;
        temp = log(temp)+3*k;
    end
    if(kStar>150)
        tempStar = 3*kStar - log(4*kStar)-0.5*log(4*pi*kStar);
    else
        tempStar = i02kStarExp-i12kStarExp;
        tempStar = log(tempStar)+3*kStar;
    end
    y = (temp - tempStar)*n;
    %%
    %calculate the second part of log ratioAcceptK_dis
    samSum = zeros(3,3);
    for i = 1:n
       samSum = samSum + dataOm(:,:,i); 
    end
    part2 = (kStar-k)*trace(s'*samSum);
    y = y + part2;
    %%
    %calcualte the third part of log ratioAcceptK_dis
    if k>150
        part3 = log(sqrt(1.5));
    else
        if k>0.01 
            R2k = i12kExp/i02kExp;
            k_kR2k = k - k*R2k;
            part3 = 0.5*log(2*k_kR2k*(1+R2k)+R2k^2-2*R2k)-log(k_kR2k)+log(k);
        else
            R2k=i12kExp/i02kExp;
            part3=0.5*log(2*k*(1+R2k)/(1-R2k)+1-1/(1-R2k)^2);
        end
    end
    if kStar>150
        part3Star = log(sqrt(1.5));
    else
        if kStar>0.01
            R2kStar = i12kStarExp/i02kStarExp;
            k_kR2kStar = kStar - kStar*R2kStar;
            part3Star = 0.5*log(2*k_kR2kStar*(1+R2kStar)+R2kStar^2-2*R2kStar)-log(k_kR2kStar)+log(kStar);
        else
            R2kStar=i12kStarExp/i02kStarExp;
            part3Star = 0.5*log(2*kStar*(1+R2kStar)/(1-R2kStar)+1-1/(1-R2kStar)^2);
        end
    end
    y=y+part3Star-part3;
    %since the value of the ratio doesn't matter when ratio>1, set it as 2
    %to avoid numerical problems if ratio>2
    if y>0
        y=2;
    else
        y=exp(y);
    end
    
function y = ratioAcceptS_or(sStar,s,k,dataOm)
    %%
    %   Synopsis:
    %       y = ratioAcceptS_dis(sStar,s,k,dataOm)
    %   Input:
    %       sStar = a proposal rotation matrix, i.e. a new proposal for s.
    %       s = the current rotation matrix.
    %       k = the current kappa which is the parameter for the random
    %       angle distribution.
    %       dataOm = the rotation matrix data. 
    %   Output:
    %       y = the ratio for accepting sStar which is a proposal for s.
    %%
    %get the sample size
    n=size(dataOm,3);
    samSum = zeros(3,3);
    %%
    %calculate density kenels
    for i = 1:n
        samSum = samSum + dataOm(:,:,i);
    end
    logRatio = k*trace((sStar-s)'*samSum);
    if(logRatio>0)
        y = 2;
    else
        y=exp(logRatio);
    end