
function mcmcSimulation(s, k, sampleSize, sInitial, kInitial, mcmcSize,simSize, tuningSK)
    %%
    %   Simulate a sample of equivalence class of rotations 
    %   and then make posterior draws of S and kappa based on the sample.
    %
    %   Synopsis:
    %       mcmcSimulation(s,k,sampleSize,sInitial, kInitial, mcmcSize, simSize, tuningSK)
    %   Input:
    %       s = the center rotation matrix;
    %       k = the parameter for the ranadom angle distribution.
    %       sampleSize = the sample size of the rotation matrix data.
    %       sInitial = the initial value for s.
    %       kInitial = the initial value for k. 
    %       mcmcSize = the size of mcmc sample.
    %       simSize = the number of mcmc samples to be simulated.
    %       tuningSK = either a 1*2 or 3*2 matrix which contains the tuning
    %           parameter for S and K. Each row of the matrix is a pair of tuning
    %           parameter for S and Kappa. If there's only one row, the fixed
    %           tuning parameters will be used in the mcmc algorithm; if there're
    %           3 rows, then the first row is the default pair of tuning parameters, the
    %           second row is the pair of tuning parameters which make the jumping rate low (specially for S) and the
    %           last row is the pair of tuning parameters which make the jumping
    %           rate high (specially for S). In the mcmc algorithm, we adjust
    %           tuning parameters if necessary. 
    %       jumpControl = an optional vector of length 3 which offers control for jumping rate when there're 3 paris
    %           of tuning parameters. The first element is the lower bound of the jumping rate, 
    %           and the second element is the upper bound of the jumping rate;
    %           the laste element is the number of step (at beginning of the mcmc algorithm) to
    %           apply the default tuning parameters. It has the default value
    %           of (0.25, 0.40, 100).

    %%
    %check passing of parameters
    %%
    %generate email subject for noticing you when exception happens
    errSubject = strcat('Exception Occus in Simulation: k=',num2str(k),', n=',num2str(sampleSize),', kInit=',num2str(kInitial),', tuningS=',num2str(tuningSK(1,1)),', tuningK=',num2str(tuningSK(1,2)));
    %%
    %create a folder
    dirName = strcat('mcmc_',num2str(k),'_',num2str(sampleSize));
    if  ~isdir(dirName) 
        safeMakeDir(dirName,errSubject);
    end
    %%
    parfor i = 1:simSize
        %generate a ratation mtrix sample
        rmSam = rmfSym(sampleSize,s,k);
        %mcmc simulation
        [kpost,spost,krate,srate]=mcmcSample(mcmcSize,sInitial,kInitial,rmSam,tuningSK);
        %---------------- write the results into files ----------------------
        %write the rotation matrix sample into a file
        rmFileName = strcat(dirName,'/rms_',num2str(k),'_',num2str(sampleSize),'_',num2str(i),'.bin');
        safeWriteBinary(rmFileName,'double',errSubject,rmSam);
        %write s into a file
        sFileName = strcat(dirName,'/spost_',num2str(k),'_',num2str(sampleSize),'_',num2str(i),'.bin');
        safeWriteBinary(sFileName,'double',errSubject,srate,spost);
        %write k into a file
        kFileName = strcat(dirName,'/kpost_',num2str(k),'_',num2str(sampleSize),'_',num2str(i),'.bin');
        safeWriteBinary(kFileName,'double',errSubject,krate,kpost);
    end
    %%
    %report progress via email
    emailSubject=strcat('100% of Simulation k=',num2str(k),', n=', num2str(sampleSize),' is Done!');
    emailBody = strcat('Tuning parameters: tuningS=',num2str(tuningSK(1,1)),', tuningK=',num2str(tuningSK(1,2)));
    %print progress message to the console
    disp(emailSubject);
    disp(emailBody);
